/**:DOC***********************************************************************/
/*                                                                           */
/* Part Name:        DFIEXALF C                                              */
/*                                                                           */
/* Descriptive Name: Alert filter exit.                                      */
/*                                                                           */
/* Copyright:        None                                                    */
/*                                                                           */
/* Status:           LAN Network Manager 1.0                                 */
/*                                                                           */
/* Function:         User customized alert filtering.                        */
/*                                                                           */
/* Entry Points:                                                             */
/*   AdapterFilter   Uses NMVT information to determine whether to pass an   */
/*                   alert to NetView or block it.                           */
/*                                                                           */
/*                                                                           */
/* 06/24/93 P5154 PRK Initialization for the char arrays are not done        */
/**:EDOC**********************************************************************/

#define         INCL_DOSSEMAPHORES
#define         PASS      0
#define         BLOCK     1
#define         MAX_SUBFIELD_LEN 512

#include        <os2.h>
#include        <string.h>
#include        <stdlib.h>

PCHAR Ebcdic2Ascii(PCHAR, PCHAR, USHORT);
PCHAR Ascii2Ebcdic(PCHAR, PCHAR, USHORT);
VOID AddrFromHex(PBYTE, PCHAR);

CHAR achAdapterAddress[13]; /* Adapter address to pass through filter */

/*****************************************************************************/
/* The variables declared in this area are all set in the filter routine to
   contain information parsed from the NMVT alert vector.  They represent much
   of the information contained in the alert vector, but not all of it.  An
   attempt was made to incude the data the user would be most likely to want
   to use to filter alerts.  Since this sample routine filters by adapter
   address, only those variables containing addresses are used in the filtering
   comparison.  The other variables are provided to allow filtering using
   different criteria. */

/* Data from subvector 0x05: hierarchy/resource list */

BYTE bResType;        /* One byte hex codepoint indicating resource type */
CHAR achResName[9];   /* Resource name */

/* Data from subvector 0x51: LAN link connection subsystem data */

CHAR achLanId[255];       /* LAN identifier */
BYTE bSegNumber[2];       /* Two byte ring or segment number */
CHAR achLocAddr[13],      /* Local individual MAC address */
     achRemAddr[13];      /* Remote individual MAC address */
BYTE bRoutingInfo[18];    /* LAN routing information - up to 18 bytes */
BYTE bBeaconData;         /* Specifies type of beaconing detected */
CHAR achUpAddr[13] ,       /* Upstream adapter MAC address */
     achDownAddr[13],     /* Downstream adapter MAC address */
     achSingAddr[13];     /* Single MAC address */
BYTE bDownWeight[2],      /* Binary severity weight for downstream MAC element*/
     bUpWeight[2],        /* Binary severity weight for upstream MAC element*/
     bBridgeId[4];        /* Bridge Identifier */
CHAR achLocName[16],      /* Local individual MAC name */
     achRemName[16],      /* Remote individual MAC name */
     achDownName[16],     /* Downstream MAC element ring fault domain name */
     achUpName[16],       /* Upstream MAC element ring fault domain name */
     achSingName[16];     /* Single MAC name */

/* Data from subvector 0x52: Link connection subsystem configuration data */

BYTE bPortAddr[2],        /* Port address of the link connection */
     bRemDevAddr,         /* DLC address of the remote link station */
     bLcsTopology,        /* LPDA-2 modem LCS topology */
     bLocDevAddr,         /* Address of the local link station */
     bLcsCorrNum[2],      /* Modem LCS correlation number */
     bLinkStatAttr[2],    /* Modem LCS link station attributes */
     bLinkConnAttr[4],    /* Modem LCS link connection attributes */
     bLslDesc;            /* LPDA fault link segment descriptor */

/* Data from subvector 0x92: Generic alert data */

BYTE bAlertType,          /* One byte codepoint indicating alert type */
     bAlertDesc[2];       /* Two byte codepoint indicating alert description */

/* Data from subvector 0x93: Probable causes */

BYTE bProbCauses[10][2];  /* Array of two byte probable cause codepoints */

/* Data from subvector 0x94: User causes */

BYTE bUserCauses[10][2],  /* Array of two byte user cause codepoints */
     bUserActions[10][2]; /* Array of two byte recommended actions codepoints */
                          /* for user causes */
CHAR achUserAddr[13];     /* Address from user causes detailed data */

/* Data from subvector 0x95: Installation causes */

BYTE bInstCauses[10][2],  /* Array of two byte installation cause codepoints */
     bInstActions[10][2]; /* Array of two byte recommended actions codepoints */
                          /* for installation causes */
CHAR achFailAddr[13],     /* Address from failure causes detailed data */
     achInstAddr[13];     /* Address from install causes detailed data */

/* Data from subvector 0x96: Failure causes */

BYTE bFailCauses[10][2],  /* Array of two byte failure cause codepoints */
     bFailActions[10][2]; /* Array of two byte recommended actions codepoints */
                          /* for failure causes */
CHAR achInstAddr[13];     /* Address from install causes detailed data */

/*****************************************************************************/

BYTE bSvLen,   /* Subvector length */
     bSvId,    /* Subvector ID */
     bSfLen,   /* Subvector subfield length */
     bSfId,    /* Subvector subfield ID */
     bLength;  /* Misc. length variable */

CHAR *pNmvtPtr,  /* Pointer to NMVT alert vector */
     *pSubField, /* Pointer to NMVT alert subfield */
     achAscii[MAX_SUBFIELD_LEN];   /* Holds strings converted from EBCDIC */

USHORT usCounter, usCount, usMvLen ; /* Counter for loops, and MvLength */

/**:DOC***********************************************************************/
/*                                                                           */
/* Entry Point: AdapterFilter                                                */
/*                                                                           */
/* Purpose:                                                                  */
/*   This function filters alerts by adapter address, allowing only alerts   */
/*   with the specified address to be passed to NetView.                     */
/*                                                                           */
/* Parameters:                                                               */
/*   pNmvt         INPUT  - PBYTE      - Pointer to NMVT.                    */
/*                                                                           */
/* Linkage:                                                                  */
/*   AdapterFilter(PBYTE pNmvt);                                             */
/*                                                                           */
/* Error Indicators:                                                         */
/*   None.                                                                   */
/*                                                                           */
/* Msgid's Processed:  None.                                                 */
/*                                                                           */
/* Special Instructions to Caller:                                           */
/*    Because this function is declared as a PASCAL function, the compiler   */
/* translates the routine's name to uppercase. LAN Network Manager forces    */
/* the routine name to uppercase when it is typed in on the Alert Filters    */
/* window. The programmer should be aware of this fact, but does not need to */
/* make any extra effort to work around it.                                  */
/*                                                                           */
/**:EDOC**********************************************************************/

USHORT far PASCAL AdapterFilter(PBYTE pNmvtPtr)
{
  BYTE abTemp[2], bRc;

/* Set address string to desired adapter address to pass */
strcpy(achAdapterAddress, "101010101010");
//VIBMPRAK PTR 5154 
achLocAddr[0] ='\0';
achRemAddr[0] ='\0';
achSingAddr[0] ='\0';
achUpAddr[0] ='\0';
achDownAddr[0] ='\0';
achUserAddr[0] ='\0';
achFailAddr[0] ='\0';
achInstAddr[0] ='\0';

/* Advance pointer past Nmvt header */
pNmvtPtr += 8;

usCount = 4;
abTemp[1]  = *pNmvtPtr;
abTemp[0]  = *(pNmvtPtr+1);
usMvLen  = *((USHORT *)abTemp);

pNmvtPtr += 4;   /* Advance the pointer to 1st subvector field */

/* Parse alert vector */
while (usCount < usMvLen)
   {
   bSvLen = *pNmvtPtr;     /* Subvector length and ID are first 2 bytes */
   bSvId = *(pNmvtPtr+1);
   switch (bSvId)
      {
      case 0x05: /* Heirarchy-resource list */
         /* This subvector (x'05') contains the resource name and type */
         pSubField = pNmvtPtr + 5;
         while (pSubField < (pNmvtPtr + bSvLen))
            {
            /* Frame text fields are in EBCDIC so... */
            Ebcdic2Ascii((pSubField+1), (PCHAR)achAscii, (USHORT)(*pSubField));
            if (!(bRc = (BYTE)memcmp(achResName, achAscii, (USHORT)(*pSubField))))
               break;
            pSubField += (CHAR) *pSubField;  /* jump to end of resource name */
            pSubField++;                     /* step over 'flags' byte */
            bResType = (BYTE) *pSubField;
            pSubField++;                     /* point to next field length */
            }
         break;

      case 0x51: /* LAN link connection subsystem data */
         /* This subvector (0x51) contains data on the elements of the LAN
            link connection */
         pSubField = pNmvtPtr+2;
         while((BYTE)(pSubField - pNmvtPtr) < bSvLen)
            {
            bSfLen = *pSubField;
            bSfId = *(pSubField+1);
            switch(bSfId)
               {
               case 0x01: /* Lan identifier */
                  memcpy(achLanId, pSubField+2, bSfLen-2);
                  break;
               case 0x02: /* Ring/segment identifier */
                  memcpy(bSegNumber, pSubField+2, 2);
                  break;
               case 0x03: /* Local individual MAC address */
                  AddrFromHex(pSubField+2,achLocAddr);
                  break;
               case 0x04: /* Remote individual MAC address */
                  AddrFromHex(pSubField+2,achRemAddr);
                  break;
               case 0x05: /* LAN routing information used by link */
                  memcpy(bRoutingInfo, pSubField+2, bSfLen-2);
                  break;
               case 0x06: /* Ring fault domain description */
                  AddrFromHex(pSubField+2,achDownAddr);
                  AddrFromHex(pSubField+8,achUpAddr);
                  break;
               case 0x07: /* Beaconing data */
                  memcpy(&bBeaconData,pSubField+2, 1);
                  break;
               case 0x08: /* Single MAC address */
                  AddrFromHex(pSubField+2,achSingAddr);
                  break;
               case 0x09: /* Fault domain error weight pair */
                  memcpy(bDownWeight, pSubField+2, 2);
                  memcpy(bUpWeight, pSubField+4, 2);
                  break;
               case 0x0A: /* Bridge identifier */
                  memcpy(bBridgeId, pSubField+2, 4);
                  break;
               case 0x23: /* Local individual MAC name */
                  memcpy(achLocName, pSubField+2, bSfLen-2);
                  break;
               case 0x24: /* Remote individual MAC name */
                  memcpy(achRemName, pSubField+2, bSfLen-2);
                  break;
               case 0x26: /* Fault domain names */
                  bLength = *(pSubField+2);
                  memcpy(achDownName, pSubField+3, bLength-1);
                  bLength = *(pSubField+2+bLength);
                  memcpy(achUpName, pSubField+3+bLength, bLength-1);
                  break;
               case 0x28: /* Single MAC name */
                  memcpy(achSingName, pSubField+2, bSfLen-2);
                  break;
               default:  /* Unrecognized subfield id */
                  break;
               } /* end switch */
            pSubField += bSfLen; /* Advance pointer to next subfield */
            } /* end while */
         break;
      case 0x52: /* Link connection subsystem configuration data */
         /* This subvector (x'52') contains data for link connections with LAN
            or LPDA-2 modems*/
         pSubField = pNmvtPtr+2;
         while((BYTE)(pSubField - pNmvtPtr) < bSvLen)
            {
            bSfLen = *pSubField;
            bSfId = *(pSubField+1);
            switch(bSfId)
               {
               case 0x01: /* Port address */
                  memcpy(bPortAddr, pSubField+2, 2);
                  break;
               case 0x02: /* Remote device address */
                  memcpy(&bRemDevAddr, pSubField+2, 1);
                  break;
               case 0x03: /* Modem LCS topology */
                  memcpy(&bLcsTopology, pSubField+2, 1);
                  break;
               case 0x04: /* Local device address */
                  memcpy(&bLocDevAddr, pSubField+2, 1);
                  break;
               case 0x05: /* Modem LCS correlation number */
                  memcpy(bLcsCorrNum, pSubField+2, 2);
                  break;
               case 0x06: /* Modem LCS link station attributes */
                  memcpy(bLinkStatAttr, pSubField+2, 2);
                  break;
               case 0x07: /* Modem LCS link attributes*/
                  memcpy(bLinkConnAttr, pSubField+2, 4);
                  break;
               case 0x08: /* LPDA fault link segment descriptor */
                  memcpy(&bLslDesc, pSubField+2, 1);
                  break;
               default:  /* Unrecognized subfield id */
                  break;
               } /* end switch */
            pSubField += bSfLen; /* Advance pointer to next subfield */
            } /* end while */
         break;
      case 0x92: /* Generic alert data */
         pSubField = pNmvtPtr+4;
         memcpy(&bAlertType, pSubField, 1);
         memcpy(bAlertDesc, pSubField+1, 2);
         break;
      case 0x93: /* Probable causes */
         usCounter = 0;
         pSubField = pNmvtPtr+2;
         while((BYTE)(pSubField - pNmvtPtr) < bSvLen)
            {
            memcpy(bProbCauses[usCounter], pSubField, 2);
            pSubField+=2;
            ++usCounter;
            } /* end while */
         break;
      case 0x94: /* User causes */
         pSubField = pNmvtPtr+2;
         while((BYTE)(pSubField - pNmvtPtr) < bSvLen)
            {
            bSfLen = *pSubField;
            bSfId = *(pSubField+1);
            switch(bSfId)
               {
               case 0x01: /* User Causes */
                  usCounter=0;
                  while(usCounter*2 < bSfLen)
                     {
                     /* Load all user cause codepoints into array */
                     memcpy(bUserCauses[usCounter],
                            pSubField+(usCounter+1)*2, 2);
                     usCounter++ ;
                     }
                  break;
               case 0x81: /* Recommended actions */
                  usCounter=0;
                  while(usCounter*2 < bSfLen)
                     {
                     /*Load all user recommended action codepoints into array */
                     memcpy(bUserActions[usCounter],
                            pSubField+(usCounter+1)*2, 2);
                     usCounter++ ;
                     }
                  break;
               case 0x82: /* Detailed data */
                  /* if subfield length is 2, no detailed data is present */
                  if (bSfLen == 2)
                     break;
                  /* if detailed data type is 0x61 detailed data is address */
                  if ( (BYTE)*(pSubField+3) == 0x61 )
                     AddrFromHex(pSubField+5,achUserAddr);
                  break;
               case 0x83: /* Product set ID index */
                  break;
               default:  /* Unrecognized subfield id */
                  break;
               } /* end switch */
            pSubField += bSfLen;
            } /* end while */
         break;
      case 0x95: /* Install causes */
         pSubField = pNmvtPtr+2;
         while((BYTE)(pSubField - pNmvtPtr) < bSvLen)
            {
            bSfLen = *pSubField;
            bSfId = *(pSubField+1);
            switch(bSfId)
               {
               case 0x01: /* Install Causes */
                  usCounter=0;
                  while(usCounter*2 < bSfLen)
                     {
                     /* Load all installation cause codepoints into array */
                     memcpy(bInstCauses[usCounter],
                            pSubField+(usCounter+1)*2, 2);
                     usCounter++ ;
                     }
                  break;
               case 0x81: /* Recommended actions */
                  usCounter=0;
                  while(usCounter*2 < bSfLen)
                     {
                     /* Load all installation recommended action codepoints */
                     /* into array */
                     memcpy(bInstActions[usCounter],
                            pSubField+(usCounter+1)*2, 2);
                     usCounter++ ;
                     }
                  break;
               case 0x82: /* Detailed data */
                  /* if subfield length is 2, no detailed data is present */
                  if (bSfLen == 2)
                     break;
                  /* if detailed data type is 0x61 detailed data is address */
                  if ( (BYTE)*(pSubField+3) == 0x61 )
                     AddrFromHex(pSubField+5,achInstAddr);
                  break;
               case 0x83: /* Product set ID index */
                  break;
               default:  /* Unrecognized subfield id */
                  break;
               } /* end switch */
            pSubField += bSfLen;
            } /* end while */
         break;
      case 0x96: /* Failure causes */
         pSubField = pNmvtPtr+2;
         while((BYTE)(pSubField - pNmvtPtr) < bSvLen)
            {
            bSfLen = *pSubField;
            bSfId = *(pSubField+1);
            switch(bSfId)
               {
               case 0x01: /* Failure Causes */
                  usCounter=0;
                  while(usCounter*2 < bSfLen)
                     {
                     /* Load all failure cause codepoints into array */
                     memcpy(bFailCauses[usCounter],
                            pSubField+(usCounter+1)*2, 2);
                     usCounter++ ;
                     }
                  break;
               case 0x81: /* Recommended actions */
                  usCounter=0;
                  while(usCounter*2 < bSfLen)
                     {
                     /* Load all failure recommended action codepoints */
                     /* into array */
                     memcpy(bFailActions[usCounter],
                            pSubField+(usCounter+1)*2, 2);
                     usCounter++ ;
                     }
                  break;
               case 0x82: /* Detailed data */
                  /* if subfield length is 2, no detailed data is present */
                  if (bSfLen == 2)
                     break;
                  /* if detailed data type is 0x61 detailed data is address */
                  if ( (BYTE)*(pSubField+3) == 0x61 )
                     AddrFromHex(pSubField+5,achFailAddr);
                  break;
               case 0x83: /* Product set ID index */
                  break;
               default:  /* Unrecognized subfield id */
                  break;
               } /* end switch */
            pSubField += bSfLen;
            } /* end while */
         break;
      default:
         break;
      } /* end switch */
   pNmvtPtr+=bSvLen; /* Advance pointer and counter to next subvector */
   usCount += bSvLen ;
   } /* end while */

/* Compare address fields from NMVT alert vector with address of adapter
   whose alerts are to be passed to NetView.  Return BLOCK or PASS to the
   LAN Network Manager alert filter */

if ( !(strcmp(achAdapterAddress, achLocAddr )) ||
     !(strcmp(achAdapterAddress, achRemAddr )) ||
     !(strcmp(achAdapterAddress, achSingAddr)) ||
     !(strcmp(achAdapterAddress, achUpAddr  )) ||
     !(strcmp(achAdapterAddress, achDownAddr)) ||
     !(strcmp(achAdapterAddress, achUserAddr)) ||
     !(strcmp(achAdapterAddress, achFailAddr)) ||
     !(strcmp(achAdapterAddress, achInstAddr)) ||
     !bRc )
   return(PASS);
else
   return(BLOCK);
}

/***************************************************************************/
/* This function converts a six byte numeric address to 12 hex characters  */
/***************************************************************************/

VOID AddrFromHex(PBYTE pHexBytes, PCHAR pCharArray)

/* pHexBytes must point to the first byte of the six byte address */
/* pCharArray must point to a 13 character array to store the hex address */

{
CHAR achStr[3]; /* String to store result of itoa function */
USHORT usCtr;   /* loop counter */

for (usCtr=0; usCtr < 6; usCtr++)
   {
   itoa( (USHORT)*(pHexBytes+usCtr), achStr, 16);
   if(achStr[1] == '\0')
      {
      achStr[1] = achStr[0];
      achStr[0] = '0';
      }
   if(achStr[0] >= 'a' && achStr[0] <= 'f') achStr[0] -= 32;
   if(achStr[1] >= 'a' && achStr[1] <= 'f') achStr[1] -= 32;
   *(pCharArray + 2*usCtr) = achStr[0];
   *(pCharArray + 2*usCtr+1) = achStr[1];
   }
*(pCharArray+12) = '\0'; /* Add null terminator */
}

/***************************************************************************/
/* Converts an EBCDIC string to ASCII, returning a pointer to the NULL     */
/* terminated ASCII string. The ASCII string must be big enough to hold    */
/* usLen+1 bytes.                                                          */
/***************************************************************************/

PCHAR Ebcdic2Ascii(PCHAR pEbcdic, PCHAR pAscii, USHORT usLen)
{
          CHAR *pReturnMe = pAscii;
   static BYTE XlatTable[] =
      {
/*         _0    _1    _2    _3    _4    _5    _6    _7    _8    _9    _A    _B    _C    _D    _E    _F */
/* 0_ */ 0x00, 0x01, 0x02, 0x03, 0xCF, 0x09, 0xD3, 0x7F, 0xD4, 0xD5, 0xC3, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
         0x10, 0x11, 0x12, 0x13, 0xC7, 0xB4, 0x08, 0xC9, 0x18, 0x19, 0xCC, 0xCD, 0x83, 0x1D, 0xD2, 0x1F,
         0x81, 0x82, 0x1C, 0x84, 0x86, 0x0A, 0x17, 0x1B, 0x89, 0x91, 0x92, 0x95, 0xA2, 0x05, 0x06, 0x07,
         0xE0, 0xEE, 0x16, 0xE5, 0xD0, 0x1E, 0xEA, 0x04, 0x8A, 0xF6, 0xC6, 0xC2, 0x14, 0x15, 0xC1, 0x1A,
         0x20, 0xA6, 0xE1, 0x80, 0xEB, 0x90, 0x9F, 0xE2, 0xAB, 0x8B, 0x9B, 0x2E, 0x3C, 0x28, 0x2B, 0x7C,
/* 5_ */ 0x26, 0xA9, 0xAA, 0x9C, 0xDB, 0xA5, 0x99, 0xE3, 0xA8, 0x9E, 0x21, 0x24, 0x2A, 0x29, 0x3B, 0x5E,
         0x2D, 0x2F, 0xDF, 0xDC, 0x9A, 0xDD, 0xDE, 0x98, 0x9D, 0xAC, 0xBA, 0x2C, 0x25, 0x5F, 0x3E, 0x3F,
         0xD7, 0x88, 0x94, 0xB0, 0xB1, 0xB2, 0xFC, 0xD6, 0xFB, 0x60, 0x3A, 0x23, 0x40, 0x27, 0x3D, 0x22,
         0xF8, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x96, 0xA4, 0xF3, 0xAF, 0xAE, 0xC5,
         0x8C, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x97, 0x87, 0xCE, 0x93, 0xF1, 0xFE,
/* A_ */ 0xC8, 0x7E, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0xEF, 0xC0, 0xDA, 0x5B, 0xF2, 0xF9,
         0xB5, 0xB6, 0xFD, 0xB7, 0xB8, 0xB9, 0xE6, 0xBB, 0xBC, 0xBD, 0x8D, 0xD9, 0xBF, 0x5D, 0xD8, 0xC4,
         0x7B, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0xCB, 0xCA, 0xBE, 0xE8, 0xEC, 0xED,
         0x7D, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0xA1, 0xAD, 0xF5, 0xF4, 0xA3, 0x8F,
         0x5C, 0xE7, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0xA0, 0x85, 0x8E, 0xE9, 0xE4, 0xD1,
/* F_ */ 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0xB3, 0xF7, 0xF0, 0xFA, 0xA7, 0xFF
      };

   while (usLen--)
      *pAscii++ = (CHAR) XlatTable[*((UCHAR *)pEbcdic++)];
   *pAscii = (CHAR) NULL;

   return pReturnMe;
}

/***************************************************************************/
/* Converts an ASCII string to EBCDIC, returning a pointer to the NULL     */
/* terminated EBCDIC string. The EBCDIC string must be big enough to hold  */
/* usLen+1 bytes.                                                          */
/***************************************************************************/

PCHAR Ascii2Ebcdic(PCHAR pAscii, PCHAR pEbcdic, USHORT usLen)
{
          CHAR *pReturnMe = pEbcdic;
   static BYTE XlatTable[] =
      {
/*         _0    _1    _2    _3    _4    _5    _6    _7    _8    _9    _A    _B    _C    _D    _E    _F */
/* 0_ */ 0x00, 0x01, 0x02, 0x03, 0x37, 0x2D, 0x2E, 0x2F, 0x16, 0x05, 0x25, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
         0x10, 0x11, 0x12, 0x13, 0x3C, 0x3D, 0x32, 0x26, 0x18, 0x19, 0x3F, 0x27, 0x22, 0x1D, 0x35, 0x1F,
         0x40, 0x5A, 0x7F, 0x7B, 0x5B, 0x6C, 0x50, 0x7D, 0x4D, 0x5D, 0x5C, 0x4E, 0x6B, 0x60, 0x4B, 0x61,
         0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0x7A, 0x5E, 0x4C, 0x7E, 0x6E, 0x6F,
         0x7C, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6,
/* 5_ */ 0xD7, 0xD8, 0xD9, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xAD, 0xE0, 0xBD, 0x5F, 0x6D,
         0x79, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96,
         0x97, 0x98, 0x99, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xC0, 0x4F, 0xD0, 0xA1, 0x07,
         0x43, 0x20, 0x21, 0x1C, 0x23, 0xEB, 0x24, 0x9B, 0x71, 0x28, 0x38, 0x49, 0x90, 0xBA, 0xEC, 0xDF,
         0x45, 0x29, 0x2A, 0x9D, 0x72, 0x2B, 0x8A, 0x9A, 0x67, 0x56, 0x64, 0x4A, 0x53, 0x68, 0x59, 0x46,
/* A_ */ 0xEA, 0xDA, 0x2C, 0xDE, 0x8B, 0x55, 0x41, 0xFE, 0x58, 0x51, 0x52, 0x48, 0x69, 0xDB, 0x8E, 0x8D,
         0x73, 0x74, 0x75, 0xFA, 0x15, 0xB0, 0xB1, 0xB3, 0xB4, 0xB5, 0x6A, 0xB7, 0xB8, 0xB9, 0xCC, 0xBC,
         0xAB, 0x3E, 0x3B, 0x0A, 0xBF, 0x8F, 0x3A, 0x14, 0xA0, 0x17, 0xCB, 0xCA, 0x1A, 0x1B, 0x9C, 0x04,
         0x34, 0xEF, 0x1E, 0x06, 0x08, 0x09, 0x77, 0x70, 0xBE, 0xBB, 0xAC, 0x54, 0x63, 0x65, 0x66, 0x62,
         0x30, 0x42, 0x47, 0x57, 0xEE, 0x33, 0xB6, 0xE1, 0xCD, 0xED, 0x36, 0x44, 0xCE, 0xCF, 0x31, 0xAA,
/* F_ */ 0xFC, 0x9E, 0xAE, 0x8C, 0xDD, 0xDC, 0x39, 0xFB, 0x80, 0xAF, 0xFD, 0x78, 0x76, 0xB2, 0x9F, 0xFF

      };

   while (usLen--)
      *pEbcdic++ = (CHAR) XlatTable[*((UCHAR *)pAscii++)];
   *pEbcdic = (CHAR) NULL;

   return pReturnMe;
}
